/*=========================================
Player Mechanics Script - By WellDoneSnake
===========================================*/

//PLAYER SCRIPT LIST:
//1) Charged Drop Dash
//2) Character Mechanics
//3) New Uncurling Behaviour
//4) Power-Up Abilities

//Hash tables storing object types that Knuckles can affect with his S.T.O.M.P
//Note: These tables actually had more content but now...
local flipmobjs = {
	[MT_SPINCUSHION] = true,
	[MT_SPRINGSHELL] = true,
	[MT_YELLOWSHELL] = true
}
local itemmobjs = {
	[MT_SIGN] = true,
	[MT_BIGTUMBLEWEED] = true,
	[MT_LITTLETUMBLEWEED] = true,
	[MT_ROLLOUTROCK] = true
}

//Jump Ability Check
addHook("AbilitySpecial", function(player)
	local mo = player.mo
	if mo.skin == "sonic" and not (mo.startdropcharge)
	and player.powers[pw_super] == 0
		S_StartSound(mo, sfx_spndsh, player)
		mo.dcounter = 0
		mo.chargedrop = true
		mo.startdropcharge = true
	end
	return false
end)

addHook("ThinkFrame", do
	for player in players.iterate
		local mo = player.mo
		//============================================
		//SCRIPT 1: "Charged Drop Dash"
		//============================================
		if mo.skin == "sonic" and mo.startdropcharge		
			if not (mo.eflags & MFE_JUSTHITFLOOR)
				if not (player.cmd.buttons & BT_JUMP) and mo.chargedrop
				or P_PlayerInPain(player) or player.playerstate & PST_DEAD
				or mo.state == S_PLAY_PAIN or player.powers[pw_carry] > 0
				or mo.eflags & MFE_SPRUNG
					mo.dcounter = 0
					mo.chargedrop = false
				elseif player.cmd.buttons & BT_JUMP and mo.chargedrop
					if mo.dcounter < player.normalspeed+(player.normalspeed/4)
						mo.dcounter = $+(2*FRACUNIT-(FRACUNIT/4))
					end
					mo.thoklayer = P_SpawnGhostMobj(mo)
					mo.thoklayer.fuse = 5
					mo.thoklayer.state = S_THOK
				end
			else
				if not P_PlayerInPain(player) and not (player.playerstate & PST_DEAD)
					if player.cmd.buttons & BT_JUMP and mo.chargedrop
						mo.state = S_PLAY_ROLL
						player.pflags = $1|PF_SPINNING
						S_StartSound(mo, sfx_zoom, player)
						P_InstaThrust(mo, mo.angle, mo.dcounter)
						mo.dcounter = 0
						mo.chargedrop = false
					end
					mo.startdropcharge = false
				end
			end
		else
			mo.dcounter = 0
			mo.chargedrop = false
			mo.startdropcharge = false
		end
	end
end)

//Shield Ability Check
addHook("ShieldSpecial", function(player)
	if not (player.mo.useshieldability) return true
	else return false end
end)

addHook("PlayerThink", function(player)
	local mo = player.mo
	if mo.skin == "sonic" or mo.skin == "knuckles"
		//Extra Script (Ability Priority):
		if not (mo.pressedcustomtwo)
			if player.cmd.buttons & BT_CUSTOM2 and not (mo.hyperthokked)
			and not (mo.shotdownwards)
				if not (mo.useshieldability)
					mo.useshieldability = true
					player.powercannotuse = true
					player.shieldicon = "SHIELD"
					player.shieldicontimer = 20
					S_StartSound(mo, sfx_s3k63)
				else
					mo.useshieldability = false
					player.powercannotuse = false
					player.shieldicon = "CROSS"
					player.shieldicontimer = 20
					S_StartSound(mo, sfx_lose)
				end
			end
		end
		//Custom 2 button press check
		mo.pressedcustomtwo = false
		if player.cmd.buttons & BT_CUSTOM2
			mo.pressedcustomtwo = true
		end
		if player.shieldicontimer != nil and player.shieldicontimer > 0
			player.shieldicontimer = $-1
		end
		
		//====================================
		//SCRIPT 2: "Character Mechanics"
		//====================================
		
		//Set change timer
		if mo.changetimer == nil
			mo.changetimer = 0
		end
		//Orb behaviour
		if mo.orb and mo.orb.valid
			if mo.orbheight == nil
				mo.orbheight = mo.z
			end
			//Orb position fixing stuff
			if P_IsObjectOnGround(mo)
				if not (mo.eflags & MFE_VERTICALFLIP)
					mo.orbheight = mo.z+6*FRACUNIT
				else
					mo.orbheight = mo.z-6*FRACUNIT
				end
			else
				mo.orbheight = mo.z-6*FRACUNIT
			end
			P_TeleportMove(mo.orb, mo.x+mo.momx, mo.y+mo.momy, mo.orbheight+mo.momz)
		end
		
		//Character Switch Behaviour (On Floor)
		if mo.charswitch
			//Actions or getting hurt cancels character switch
			if player.pflags & PF_JUMPED or player.pflags & PF_SPINNING
			or P_PlayerInPain(player)
				mo.stk = false
				mo.kts = false
				mo.changetimer = 0
				mo.ghost = nil
				mo.charswitch = false
				return
			end		
			//Timer increase prompts character switch animation
			if not (mo.timerdecrease)
				mo.changetimer = $+1
				if mo.changetimer > 4
					if mo.skin == "sonic"
						R_SetPlayerSkin(player, "knuckles")
						if player.powers[pw_super] > 0
							mo.color = 72
						end
					elseif mo.skin == "knuckles"
						R_SetPlayerSkin(player, "sonic")
						if player.powers[pw_super] > 0
							mo.color = 82
						end
					end
					mo.timerdecrease = true
				end	
			elseif mo.timerdecrease
				mo.changetimer = $-1
				if mo.changetimer < 0
					mo.stk = false
					mo.kts = false
					mo.changetimer = 0
					mo.charswitch = false
				end
			end
			//Keep walking and running animations
			if mo.state == S_PLAY_WALK
				if mo.framechange < 7
					mo.framechange = $+1
				else
					mo.framechange = 0
				end
			elseif mo.state == S_PLAY_RUN
				if player.powers[pw_super] > 0 and mo.skin == "sonic"
					if mo.framechange < 1
						mo.framechange = $+1
					else
						mo.framechange = 0
					end
				else
					if mo.framechange < 3
						mo.framechange = $+1
					else
						mo.framechange = 0
					end
				end
			else
				mo.framechange = 0
			end
			//Spin around baby!
			if mo.stk and mo.changetimer != 4
				mo.spinangle = $+FixedAngle(45*FRACUNIT)
			elseif mo.kts and mo.changetimer != 4
				mo.spinangle = $-FixedAngle(45*FRACUNIT)
			end
			player.drawangle = mo.spinangle
			mo.frame = mo.changetimer<<FF_TRANSSHIFT|mo.framechange
		end
		
		//Knuckles Glide Ghost Behaviour
		if mo.knucklesghost and mo.knucklesghost.valid
			local kghost = mo.knucklesghost
			local ghostofghost = P_SpawnGhostMobj(kghost) -- Afterimage
			ghostofghost.fuse = 3
			if kghost.target and kghost.target.valid
				if not (mo.shotknuxmissile)
					kghost.momx = (mo.x - kghost.x) + FixedMul(cos(kghost.angle-ANGLE_90), 80*FRACUNIT)
					kghost.momy = (mo.y - kghost.y) + FixedMul(sin(kghost.angle-ANGLE_90), 80*FRACUNIT)
					kghost.momz = mo.z - kghost.z
					kghost.angle = $+ANGLE_22h/2
				else
					kghost.tracer = mo.enemy
					if kghost.tracer and kghost.tracer.valid
						A_HomingChase(kghost, 18*FRACUNIT, 1)
					end
				end
				if P_IsObjectOnGround(mo) and not (mo.shotknuxmissile)
					if kghost and kghost.valid
						kghost.state = S_NULL
					end
				end
			end
		end

		if mo.enemy and mo.enemy.valid
			if not (player.playerstate & PST_DEAD)
			and not P_PlayerInPain(player) and not (mo.shotknuxmissile)
				if not (mo.buttonpress) and player.cmd.buttons & BT_CUSTOM1
					if mo.knucklesghost and mo.knucklesghost.valid
						mo.knucklesghost.tracer = mo.enemy
						mo.knucklesghost.flags = $^^MF_MISSILE
						mo.knucklesghost.flags = $&~(MF_NOCLIP|MF_NOCLIPHEIGHT)
						mo.knucklesghost.fuse = 2*TICRATE
						mo.shotknuxmissile = true
					end
				end
			end
		end

		/*Do not allow character switching if any of the following conditions
		return true.*/
		if not (mo.buttonpress) and player.exiting == 0
		and not P_IsObjectInGoop(mo) and not (player.playerstate & PST_DEAD)
		and not (player.pflags & PF_STARTDASH)
			
			//Enemy Search & Lock-On
			if not P_IsObjectOnGround(mo) and not (player.pflags & PF_SPINNING)
			and not (mo.eflags & MFE_SPRUNG) and player.powers[pw_carry] == 0
			and mo.state == S_PLAY_JUMP and player.pflags & PF_JUMPED
			and not (mo.hyperthokked) and not (mo.shotdownwards)
				if mo.enemy == nil
					//Roundabout enemy check to avoid Knuckles missile attack
					//occuring when there's a boss and attack it.
					mo.targetenemy = P_LookForEnemies(player, false)
					if mo.targetenemy != nil
						local mobj = mo.targetenemy
						if not (mobj.flags & MF_BOSS)
						and mobj.type != MT_BIGMINE
						and mobj.type != MT_ROSY
						and mobj.type != MT_EGGSHIELD
							mo.enemy = mo.targetenemy
						else
							mo.targetenemy = nil
						end		
					end
				elseif mo.enemy and mo.enemy.valid
					local direction = R_PointToAngle2(mo.x, mo.y, mo.enemy.x, mo.enemy.y)-mo.angle
					if direction > ANGLE_45 or direction < ANGLE_315
					or mo.enemy.z < mo.z-160*FRACUNIT
						mo.enemy = nil
					else
						if mo.enemy.health > 0 and not (mo.specialattacked)
							P_SpawnLockOn(player, mo.enemy, S_LOCKON1)
						end
					end
				end
			else
				mo.enemy = nil
			end
		
			//On ground change
			if P_IsObjectOnGround(mo) and not (player.pflags & PF_SPINNING)
			and not (mapheaderinfo[gamemap].typeoflevel & TOL_NIGHTS)
			and mo.state != S_PLAY_GLIDE_LANDING and mo.state != S_PLAY_GLIDE
				if player.cmd.buttons & BT_CUSTOM1
					mo.timerdecrease = false
					mo.spinangle = player.drawangle
					mo.framechange = mo.frame
					mo.charswitch = true
					if mo.skin == "sonic"
						mo.stk = true
						mo.kts = false
					elseif mo.skin == "knuckles"
						mo.kts = true
						mo.stk = false
					end
					if mo.orb and mo.orb.valid
						mo.orb.state = S_NULL
					end
					mo.orb = P_SpawnMobj(mo.x+mo.momx, mo.y+mo.momy, mo.z+mo.momz, MT_CHARACTERORB)
					S_StartSoundAtVolume(mo.origin, player.seesound, 120, player)
				end
			//Mid-air change
			elseif not P_IsObjectOnGround(mo) and player.pflags & PF_JUMPED
			and not mo.hyperthokked and not mo.shotdownwards
			and not (player.pflags & PF_GLIDING)
			and mo.state < S_PLAY_SUPER_TRANS1 or mo.state > S_PLAY_SUPER_TRANS6
				if player.cmd.buttons & BT_CUSTOM1 and not (mo.airswitch)
					if mo.orb and mo.orb.valid
						mo.orb.state = S_NULL
					end
					mo.orb = P_SpawnMobj(mo.x+mo.momx, mo.y+mo.momy, mo.z+mo.momz, MT_CHARACTERORB)
					if mo.skin == "sonic"
						mo.sonicghost = P_SpawnGhostMobj(mo)
						mo.sonicghost.fuse = 15
						P_SpawnThokMobj(player)
						R_SetPlayerSkin(player, "knuckles")
						if mo.enemy and mo.enemy.valid
							mo.homingattacktimer = 3*TICRATE
							mo.homingattacked = true
						else
							mo.homingattacked = false
							P_SetObjectMomZ(mo, 4*FRACUNIT, false)
							P_InstaThrust(mo, mo.angle, 36*FRACUNIT)
						end
						S_StartSoundAtVolume(mo.origin, sfx_thok, 255, player)
					elseif mo.skin == "knuckles" and not (player.pflags & PF_GLIDING)
						mo.shotknuxmissile = false
						//Ghost 1 - Spawn Ghost
						mo.knuxdummyghost = P_SpawnGhostMobj(mo)
						mo.knuxdummyghost.fuse = 15
						//Ghost 2 - Ghost in Orbit
						if mo.knucklesghost and mo.knucklesghost.valid -- An existing mobj?
							mo.knucklesghost.state = S_NULL -- Get rid of it
						end
						mo.knucklesghost = P_SpawnGhostMobj(mo)
						//mo.knucklesghost.flags = $1^^MF_MISSILE
						mo.knucklesghost.state = S_PLAY_GLIDE
						mo.knucklesghost.frame = TR_TRANS20
						mo.knucklesghost.fuse = 5*TICRATE
						mo.knucklesghost.target = mo

						R_SetPlayerSkin(player, "sonic")
						P_SetObjectMomZ(mo, 8*FRACUNIT, false)
						S_StartSoundAtVolume(mo.origin, sfx_wdjump, 255, player)
					end
					mo.airswitch = true -- player cannot switch again
				end
			end
			/*After switching in mid-air and then touching the floor, the
			player should be able to switch characters once again*/
			if P_IsObjectOnGround(mo) or mo.climbed or mo.carried
			or mo.eflags & MFE_SPRUNG
				mo.specialattacked = false
				mo.airswitch = false
				mo.startdropcharge = false
			end	
		end
		
		if mo.homingattacked
			if not (player.playerstate & PST_DEAD)
			and not P_PlayerInPain(player) and not (mo.eflags & MFE_SPRUNG)
			and mo.state == S_PLAY_JUMP and mo.homingattacktimer > 0
				if mo.enemy and mo.enemy.valid
					if mo.enemy.health > 0
						mo.homingattacktimer = $-1
						P_HomingAttack(mo, mo.enemy)
					else
						P_SetObjectMomZ(mo, 8*FRACUNIT, false)
						P_InstaThrust(mo, mo.angle, 8*FRACUNIT)
						mo.homingattacked = false
					end
				else
					mo.homingattacked = false
				end
			else
				mo.homingattacked = false
			end
		end
		
		//Button press check
		mo.buttonpress = false
		if player.cmd.buttons & BT_CUSTOM1
			mo.buttonpress = true
		end
		
		//===============================
		//SCRIPT 3: "Uncurling"
		//===============================
		
		//Uncurling checks
		if player.cmd.buttons & BT_USE and mo.prevstate == S_PLAY_ROLL
		and player.pflags & PF_SPINNING
		and not (player.pflags & PF_STARTDASH)
		and not (player.pflags & PF_JUMPED)
		and mo.tracer == nil -- Prevent player from unspinning in a zoom tube
			mo.unspinattempt = true
			mo.unspintimer = $-1 -- Decrement timer
		elseif not (player.cmd.buttons & BT_USE)
		
			/*Prevents starting the unspintimer when switching from
			walking/running to spinning. The player's previous state is stored
			into a variable and compared to their current states when holding
			down the spin button. EDIT: For some reason this part of the
			script still works srb2 2.2*/
			mo.prevstate = mo.state -- Use this as a pseudo-button-press check
			
			if mo.unspinattempt -- Attempted to unspin?
				if mo.unspintimer > 25 -- Player can unspin within 10 tics
					player.pflags = $&~PF_SPINNING
					
					//Switch to run or fall states when uncurling
					if P_IsObjectOnGround(mo)
						mo.state = S_PLAY_RUN
					else
						mo.state = S_PLAY_FALL
					end
				end
			end
			//Reset variables
			mo.unspinattempt = false
			mo.unspintimer = TICRATE
		end
		
		//===============================
		//SCRIPT 4: "Power-Up Abilities"
		//===============================
		
		//Sonic's Abilities
		if mo.skin == "sonic" and player.spc >= 1
			player.normalspeed = 405*FRACUNIT/10 -- Speed Boost(+12.5%)
		end
		
		//Hyperthok Mechanic
		if player.spc > 1
			if not (mo.useshieldability)
				if mo.skin == "sonic"
				and mo.state < S_PLAY_SUPER_TRANS1 or mo.state > S_PLAY_SUPER_TRANS6
					if mo.htp == nil -- "htp" = Hyper Thok Power
						mo.htp = 0
					end
					if not (mo.thokcharged)
						if P_IsObjectOnGround(mo) and player.pflags & PF_STARTDASH
						and player.cmd.buttons & BT_USE and player.cmd.buttons & BT_CUSTOM1
							mo.htp = $+1
							if mo.htp == 1
								mo.spinghost = P_SpawnGhostMobj(mo)
								mo.spinghost.fuse = 74
							else
								if mo.spinghost and mo.spinghost.valid
									P_TeleportMove(mo.spinghost, mo.x, mo.y, mo.z)
									mo.spinghost.scale = $+(mo.htp*FRACUNIT)/600
									mo.spinghost.angle = mo.angle
								end
							end
							if mo.htp >= 75
								P_ResetPlayer(player)
								mo.state = S_PLAY_STND
								mo.thokcharged = true
							end
						else
							mo.htp = 0
							if mo.spinghost and mo.spinghost.valid
								mo.spinghost.state = S_NULL
							end
						end
					else
						if player.cmd.buttons & BT_USE and mo.thokcharged
						and not P_IsObjectOnGround(mo)
						and player.pflags & PF_JUMPED
						and not (player.pflags & PF_JUMPDOWN)
						and not P_PlayerInPain(player)
						and not (player.playerstate & PST_DEAD)
							mo.momz = 0
							//Shoot the player at 2x the hyperthokpower's value
							P_InstaThrust(mo, mo.angle, (mo.htp*FRACUNIT)*2)
							S_StartSound(mo.origin, player.activesound, player)
							mo.htp = 0
							mo.hyperthokked = true
							mo.thokcharged = false
						end
					end
					if mo.hyperthokked
						if not P_IsObjectOnGround(mo)
							P_SpawnThokMobj(player)
							P_SpawnGhostMobj(mo)
						else
							mo.hyperthokked = false
						end
					end
				else -- Charge is lost when switching to Knuckles
					mo.htp = 0
					mo.hyperthokked = false
					mo.thokcharged = false
				end
			end
		end

		//Knuckles' Abilities
		if mo.skin == "knuckles" and player.kpc >= 1
			player.jumpfactor = (FRACUNIT/10)*11 -- Jump Boost(+approx. 25%)
			player.actionspd = 30*FRACUNIT -- Glide Boost(+5*FRACUNIT)
		end
		
		//Checks that prevent Knuckles from plummeting to the earth when letting go
		//off of something.
		if player.climbing == 1
			mo.climbed = true
		else
			if mo.climbed
				if player.cmd.buttons & BT_USE
					mo.cancelclimb = true
				elseif player.cmd.buttons & BT_JUMP
					mo.cancelclimb = false
					mo.cancelcarry = false
				end
				mo.climbed = false
			end
		end
		if player.powers[pw_carry] > 0
			mo.carried = true
		else
			if mo.carried
				if player.cmd.buttons & BT_USE
					mo.cancelcarry = true
				elseif player.cmd.buttons & BT_JUMP
					mo.cancelclimb = false
					mo.cancelcarry = false
				end
				mo.carried = false
			end
		end
		if mo.cancelclimb or mo.cancelcarry
			if P_IsObjectOnGround(mo)
				mo.cancelclimb = false
				mo.cancelcarry = false
			end
		end
			
		
		//S.T.O.M.P Mechanic
		if player.kpc > 1
			if not (mo.useshieldability)
				if mo.skin == "knuckles"
				and mo.state < S_PLAY_SUPER_TRANS1 or mo.state > S_PLAY_SUPER_TRANS6
					if not (mo.shotdownwards)
						if not P_IsObjectOnGround(mo) and player.pflags & PF_JUMPED
						and player.cmd.buttons & BT_USE and not (player.cmd.buttons & BT_JUMP)
						and not (player.pflags & PF_GLIDING) and not (mo.cancelclimb)
						and not (mo.cancelcarry)
							mo.rangle = 0
							mo.momx = 0
							mo.momy = 0
							mo.shotdownwards = true
						end
					else
						if mo.eflags & MFE_SPRUNG
							mo.shotdownwards = false
						end
						if P_IsObjectOnGround(mo)
							S_StartSound(mo.origin, sfx_earth, player)
							P_StartQuake(8*FRACUNIT, TICRATE/2)
							
							//"P_RadiusCheck" is weird, gonna search the blockmap
							//differently.
							searchBlockmap("objects", function(mo, mobj)
								if mobj.z <= mo.z+128*FRACUNIT and mobj.z >= mo.z-128*FRACUNIT
									if mobj.flags & MF_ENEMY
										if mobj.type == MT_SNAILER
											P_DamageMobj(mobj, mo, mo, 1)
										elseif mobj.type == MT_EGGGUARD
										and mobj.state >= S_EGGGUARD_RUN1
										and mobj.state <= S_EGGGUARD_RUN4
											P_DamageMobj(mobj, mo, mo, 1)
										elseif flipmobjs[mobj.type] and P_IsObjectOnGround(mobj)
											if not (mobj.flipped)
												mobj.shiftnum = 0
												mobj.flipped = true
												mobj.unflipped = false		
											else
												mobj.flipped = false
												mobj.unflipped = true
											end
											P_SetObjectMomZ(mobj, 12*FRACUNIT, false)
										elseif mobj.type == MT_JETJAW
											mo.pushangle = R_PointToDist2(mobj.x, mobj.y, mo.x, mo.y)
											P_InstaThrust(mobj, mo.pushangle, -24*FRACUNIT)
										else
											if P_IsObjectOnGround(mobj) and not (mobj.type == MT_INVISIBLESPRINGSHELL)
											or mobj.type == MT_SKIM or mobj.type == MT_BIGMINE
												P_DamageMobj(mobj, mo, mo, 1)
											end
										end
									elseif itemmobjs[mobj.type] or mobj.type >= 132 and mobj.type <= 137
									or mobj.flags & MF_MONITOR
										P_SetObjectMomZ(mobj, 6*FRACUNIT, false)
									elseif mobj.type == MT_TNTBARREL or mobj.type == MT_EGGSHIELD
										P_KillMobj(mobj)
									end
								end
							end, mo, mo.x-144*FRACUNIT, mo.x+144*FRACUNIT, mo.y-144*FRACUNIT, mo.y+144*FRACUNIT)
							local debris
							if debris == nil
								debris = 0
							end
							for i = 0, 31
								local xpos = mo.x+FixedMul(64*FRACUNIT, cos(mo.rangle))
								local ypos = mo.y+FixedMul(64*FRACUNIT, sin(mo.rangle))
								if not (mo.eflags & MFE_UNDERWATER) and not (mo.eflags & MFE_TOUCHWATER)
									debris = P_SpawnMobj(xpos, ypos, mo.z, MT_DUST)
								else
									debris = P_SpawnMobj(xpos, ypos, mo.z, MT_BUBBLETHING)
								end
								debris.scale = 13*FRACUNIT/10
								mo.rangle = $+FixedAngle(1125*FRACUNIT/100)
								if debris and debris.valid
									if not (debris.flagset)
										if not (debris.eflags & MFE_UNDERWATER)
											debris.flags = $&~(MF_NOGRAVITY|MF_NOCLIP|MF_NOCLIPHEIGHT)
											debris.flagset = true
										end
									end
									P_InstaThrust(debris, mo.rangle, 12*FRACUNIT)
								end
							end
						end
						if not P_IsObjectOnGround(mo) and not P_PlayerInPain(player)
						and not (player.playerstate & PST_DEAD) and not P_IsObjectInGoop(mo)
						and player.powers[pw_carry] == 0
							P_SetObjectMomZ(mo, -36*FRACUNIT, false)
							P_SpawnThokMobj(player)
							P_SpawnGhostMobj(mo)
						else
							mo.landtimer = 20
							mo.shotdownwards = false
						end
					end
				else
					mo.shotdownwards = false
				end
			end
		end
		if not P_IsObjectInGoop(mo) and not P_PlayerInPain(player)
		and not (player.playerstate & PST_DEAD) and player.powers[pw_carry] == 0
			if mo.landtimer != nil and mo.landtimer > 0
				mo.landtimer = $-1
				mo.state = S_PLAY_GLIDE_LANDING
				player.pflags = $1|PF_FULLSTASIS
			end
		else
			if mo.landtimer != nil and mo.landtimer > 0
				if P_IsObjectInGoop(mo)
				or player.powers[pw_carry] == CR_ZOOMTUBE
					mo.state = S_PLAY_ROLL
				elseif player.powers[pw_carry] == CR_PLAYER
				or player.powers[pw_carry] == CR_ROPEHANG
					mo.state = S_PLAY_CLING
				elseif player.powers[pw_carry] == CR_MACESPIN
					mo.state = S_PLAY_JUMP
				elseif player.powers[pw_carry] == CR_PTERABYTE
					mo.state = S_PLAY_FALL
				elseif player.powers[pw_carry] == CR_MINECART
				or player.powers[pw_carry] == CR_BRAKGOOP
					mo.state = S_PLAY_STND
				end
				mo.landtimer = 0
			end
		end
		//Misc.
		player.seesound = sfx_whoosh
	else
		mo.useshieldability = true
	end
end)