/*Custom Floating behaviour
Var1 controls the speed at which the actor moves up & down
- default value: "2"
Var2 determines the actor's minimum and maximum z-momentum
- default value: "1"*/
function A_CustomFloat(actor, var1, var2)
	if var1 == nil or var1 == 0
		var1 = 2
	end
	if var2 == nil or var2 == 0
		var2 = 1
	end
	if leveltime % var1 == 0
		if actor.momz < (var2-(var2*2))*FRACUNIT
			actor.floating = false
		elseif actor.momz > var2*FRACUNIT
			actor.floating = true
		end
		if not (actor.floating)
			actor.momz = $+FRACUNIT
		elseif (actor.floating)
			actor.momz = $-FRACUNIT
		end
	end
end

/*Fading behaviour - the actor becomes more transparent/translucent each time
this action is called:
- Var1 = 0 - actor will become less transparent each tic
- Var1 > 0 - actor will become more transparent each tic
- Var2 determines the state to go to after the fading effect is completed:
0 = SeeState, 1 = MeleeState, 2 = MissileState, 3 = DeathState,
4 = XDeathState, 5 = RaiseState,
6 = The next state (next state number after current state not to be confused
with "NextState" variable)*/
function A_FadeEffect(actor, var1, var2)
	if actor.trans == nil
		if var1 == 0
			actor.fadein = true
			actor.trans = 9
		else
			actor.fadeout = true
			actor.trans = 0
		end
	end
	if actor.fadein
		actor.trans = $-1
	elseif actor.fadeout
		actor.trans = $+1
	end
	
	actor.frame = actor.trans<<FF_TRANSSHIFT|$
	if actor.fadein and actor.trans < 0
	or actor.fadeout and actor.trans > 9
	or actor.fadeout and actor.flags & MF_BOSS and actor.trans > 6
		actor.trans = nil
		actor.fadein = false
		actor.fadeout = false
		if var2 == 0
			actor.state = actor.info.seestate
		elseif var2 == 1
			actor.state = actor.info.meleestate
		elseif var2 == 2
			actor.state = actor.info.missilestate
		elseif var2 == 3
			actor.state = actor.info.deathstate
		elseif var2 == 4
			actor.state = actor.info.xdeathstate
		elseif var2 == 5
			actor.state = actor.info.raisestate
		elseif var2 == 6
			actor.state = $+1
		end
	end
end

//Set/Reset the appropriate variables on spawn (map dependent)
local function PlayerAbilityReset(player)
	//Revert player's stats
	if player.mo.skin == "sonic"
		player.normalspeed = 36*FRACUNIT
	elseif player.mo.skin == "knuckles"
		player.jumpfactor = (85*FRACUNIT)/100
		player.actionspd = 25*FRACUNIT
	end
	player.hyperthok = false
	player.earthbreaker = false
	//Don't show power-up HUD in boss stages and NiGHTS stages
	if mapheaderinfo[gamemap].bonustype != 0 or G_IsSpecialStage()
		player.poweruphud = false
	else
		player.poweruphud = true
	end
end
addHook("PlayerSpawn", PlayerAbilityReset)

//Modified Behaviour for Sonic
freeslot("sfx_hypthk", "sfx_earth")

addHook("PlayerThink", function(player)
	if player.mo.skin == "sonic"
		if player.powers[pw_super] == 0
			player.charability = CA_JUMPBOOST
			player.actionspd = 10*FRACUNIT
		else
			player.charability = CA_THOK
			player.actionspd = 60*FRACUNIT
		end
		player.jumpfactor = (FRACUNIT/10)*9
		player.activesound = sfx_hypthk
	end
end)

//Orb that forms around characters when switching
freeslot("MT_CHARACTERORB", "S_CORB", "sfx_whoosh")
mobjinfo[MT_CHARACTERORB] = {
	doomednum = -1,
	spawnstate = S_CORB,
	spawnhealth = 1,
	dispoffset = 1,
	flags = MF_SCENERY|MF_NOCLIP|MF_NOCLIPHEIGHT|MF_NOGRAVITY
}
states[S_CORB] = {SPR_PITY, TR_TRANS70|FF_ANIMATE|A, 9, nil, 9, 1, S_NULL}

//Ensuring that Knuckles Missile isn't overpowered and rewards the player
//a point towards their kill count.
for i = 0, #mobjinfo-1 do
	if not (mobjinfo[i].flags & MF_ENEMY)
		continue
	end
	addHook("MobjDamage", function(mobj, kghost, mo)
		if mo and mo.valid and kghost and kghost.valid
			if kghost.type == MT_GHOST
				kghost.target = mo
				kghost.fuse = 4
				P_DamageMobj(mobj, mo, mo, 1) -- set source to "mo" to avoid C stack overflow
				return true
			end
		end
	end, i)
end

//Bubbles that spawn when you S.T.O.M.P underwater
freeslot("MT_BUBBLETHING", "S_BUBBLEBUBBLE")
mobjinfo[MT_BUBBLETHING] = {
	doomednum = -1,
	spawnstate = S_BUBBLEBUBBLE,
	speed = 8,
	reationtime = 8,
	radius = 4*FRACUNIT,
	height = 4*FRACUNIT,
	spawnhealth = 1000,
	mass = 16,
	dispoffset = 0,
	flags = MF_SCENERY
}
states[S_BUBBLEBUBBLE] = {SPR_BUBO, A, 1, nil, 0, 0, S_MEDIUMBUBBLE}

addHook("MobjThinker", function(mobj)
	if mobj.timer == nil
		mobj.timer = 3*TICRATE
	end
	if mobj.timer > 0
		mobj.timer = $-1
	else
		mobj.state = S_NULL
	end
end, MT_BUBBLETHING)


//========================================================================
//The following stuff below all relates to Knuckles' S.T.O.M.P ability.
//========================================================================

freeslot("MT_INVISIBLESPRINGSHELL")
mobjinfo[MT_INVISIBLESPRINGSHELL] = {
	doomednum = 1,
	spawnstate = S_INVISIBLE,
	spawnhealth = 1,
	reactiontime = 32,
	deathsound = S_XPLD1,
	deathsound = sfx_pop,
	speed = 6,
	radius = 24*FRACUNIT,
	height = 40*FRACUNIT,
	flags = MF_ENEMY|MF_SPECIAL|MF_SHOOTABLE
}
local function FlipShell(mobj)
	if mobj.flipped and not (mobj.newmoves)
		if mobj.shellcopy == nil
			mobj.shellcopy = P_SpawnMobj(mobj.x, mobj.y, mobj.z, MT_INVISIBLESPRINGSHELL)
			mobj.flags = $&~(MF_ENEMY|MF_SPECIAL|MF_SHOOTABLE)
		end
		if mobj.shiftnum < 180*FRACUNIT
			mobj.shiftnum = $+10*FRACUNIT
			mobj.rollangle = FixedAngle(mobj.shiftnum)
		end
		if mobj.momz <= 0 and P_IsObjectOnGround(mobj) and not (mobj.newmoves)
			mobj.movetimer = 7
			mobj.newmoves = true
		end
	end
	if mobj.newmoves
		if P_IsObjectOnGround(mobj)
			mobj.movetimer = 7
			P_SetObjectMomZ(mobj, 12*FRACUNIT, false)
			S_StartSound(mobj, mobj.info.painsound)
		else
			if mobj.movetimer > 4
				if leveltime % 4 == 0
					mobj.movetimer = $-1
				end
			elseif mobj.movetimer == 4
				mobj.movetimer = 0
			end
		end
		mobj.frame = mobj.movetimer
	end
	if mobj.shellcopy and mobj.shellcopy.valid
		if mobj.shellcopy.health > 0
			P_TeleportMove(mobj.shellcopy, mobj.x+mobj.momx, mobj.y+mobj.momy, mobj.z+mobj.momz)
		end
	else
		if not (mobj.flags & MF_ENEMY) and not (mobj.dying)
			mobj.newmoves = false
			mobj.momz = 0
			mobj.rollangle = 0
			P_KillMobj(mobj)
			mobj.dying = true
		end
	end
end
addHook("MobjThinker", FlipShell, MT_SPRINGSHELL)
addHook("MobjThinker", FlipShell, MT_YELLOWSHELL)

addHook("MobjThinker", function(mobj)
	if mobj.shiftnum == nil
		mobj.shiftnum = 0
	end
	if mobj.flipped and mobj.health > 0
		mobj.momx = 0
		mobj.momy = 0
		if mobj.shiftnum < 180*FRACUNIT
			mobj.shiftnum = $+10*FRACUNIT
		end	
		if mobj.momz <= 0 and P_IsObjectOnGround(mobj)
			return true
		end
	elseif mobj.unflipped
		if mobj.shiftnum > 0
			mobj.shiftnum = $-10*FRACUNIT
		end
	end
	if mobj.health > 0
		mobj.rollangle = FixedAngle(mobj.shiftnum)
	else
		mobj.momz = 0
		mobj.rollangle = 0
	end
end, MT_SPINCUSHION)

addHook("ShouldDamage", function(mo, mobj)
	if mobj and mobj.valid
		local player = mo.player
		if mobj.type == MT_SPINCUSHION and mobj.flipped
			if mo.state == S_PLAY_JUMP
				if mobj.state >= S_SPINCUSHION_SPIN1 and mobj.state <= S_SPINCUSHION_SPIN4
					S_StartSound(mobj, sfx_s3k42)
					local an = R_PointToAngle2(mo.x, mo.y, mobj.x, mobj.y)
					P_InstaThrust(mo, an, -player.speed+-8*FRACUNIT)
				end
				return false
			else
				return true
			end
		end
	end
end, MT_PLAYER)

addHook("MobjDamage", function(mobj, mo)
	if mo and mo.valid
		if not (mo.player) return end
		local player = mo.player
		if mobj.flipped and player.pflags & PF_SPINNING
			P_DamageMobj(mo, mobj, mobj, 1)
			return true			
		end
	end
end, MT_SPINCUSHION)